use crate::prelude::*;
use parking_lot::Mutex;
use std::sync::Arc;
use teloxide::types::InputFile;
use tokio::task::spawn_blocking;

/// Decryptes the account on a worker thread and adds it to the accounts vector
#[inline]
async fn decrypt_account(
    account: account::Model,
    master_pass: Arc<str>,
    accounts: &Mutex<&mut Vec<DecryptedAccount>>,
) -> crate::Result<()> {
    let account =
        spawn_blocking(move || DecryptedAccount::from_account(account, &master_pass)).await??;
    accounts.lock().push(account);
    Ok(())
}

/// Gets the master password, decryptes the account and sends the json file to the user
async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    master_pass: String,
) -> crate::Result<()> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    let mut accounts = Vec::new();

    {
        let accounts = Mutex::new(&mut accounts);
        let master_pass: Arc<str> = master_pass.into();

        Account::get_all(user_id, &db)
            .await?
            .err_into::<crate::Error>()
            .try_for_each_concurrent(3, |account| {
                decrypt_account(account, master_pass.clone(), &accounts)
            })
            .await?;
    }

    accounts.sort_unstable_by(|this, other| this.name.cmp(&other.name));

    let json = spawn_blocking(move || serde_json::to_vec_pretty(&User { accounts })).await??;
    let file = InputFile::memory(json).file_name("accounts.json");

    bot.send_document(msg.chat.id, file)
        .reply_markup(deletion_markup())
        .await?;
    dialogue.exit().await?;
    Ok(())
}

/// Handles /export command
pub async fn export(bot: Throttle<Bot>, msg: Message, dialogue: MainDialogue) -> crate::Result<()> {
    let previous = bot
        .send_message(msg.chat.id, "Send a master password to export the accounts")
        .await?;
    dialogue
        .update(State::GetMasterPass(Handler::new(
            get_master_pass,
            &previous,
        )))
        .await?;
    Ok(())
}
